<?php

namespace Drupal\eca_parameters\EventSubscriber;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\eca\EventSubscriber\EcaExecutionSubscriberBase;
use Drupal\eca_parameters\Event\EcaParametersEvents;
use Drupal\eca_parameters\Event\RequestingParameterEvent;
use Drupal\parameters\Entity\ParametersCollectionInterface;
use Drupal\parameters\Event\CollectionsPreparationEvent;
use Drupal\parameters\Event\ParameterEvents;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

/**
 * ECA Parameters event subscriber.
 */
class EcaParameters extends EcaExecutionSubscriberBase {

  /**
   * The parameters collection ID to additionally use.
   *
   * @var string
   */
  protected static string $collectionId = 'eca';

  /**
   * The event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected EventDispatcherInterface $eventDispatcher;

  /**
   * Invoked when the collection preparation event dispatches.
   *
   * @param \Drupal\parameters\Event\CollectionsPreparationEvent $event
   *   The event.
   */
  public function onPreparation(CollectionsPreparationEvent $event): void {
    if (isset($event->name)) {
      // A parameter name is specified, dispatch an explicit event for it.
      $this->eventDispatcher->dispatch(new RequestingParameterEvent($event), EcaParametersEvents::REQUEST);
    }
    try {
      $storage = $collection = $this->entityTypeManager->getStorage(ParametersCollectionInterface::ENTITY_TYPE_ID);
    }
    catch (InvalidPluginDefinitionException | PluginNotFoundException) {
      return;
    }
    // Place the ECA collection into the list of available collections.
    // It must be of higher priority than the global collection.
    if (isset(static::$collectionId) && ($storage->load(static::$collectionId))) {
      $global_collection = NULL;
      if (($last_collection = end($event->collections)) && $last_collection->id() === 'global') {
        $global_collection = array_pop($event->collections);
      }
      if (!in_array($collection, $event->collections, TRUE)) {
        $event->collections[] = $collection;
      }
      if ($global_collection) {
        $event->collections[] = $global_collection;
      }
      // Make sure the collections list starts at the beginning later on.
      reset($event->collections);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      ParameterEvents::COLLECTIONS_PREPARATION => ['onPreparation'],
    ];
  }

  /**
   * Set the event dispatcher.
   *
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $dispatcher
   *   The event dispatcher.
   */
  public function setEventDispatcher(EventDispatcherInterface $dispatcher): void {
    $this->eventDispatcher = $dispatcher;
  }

}
